#include "neuralnet.h"

#ifndef KMEANSLIB_H
#define KMEANSLIB_H

#ifdef __cplusplus


class Kmeans : public NeuralNet {

	public :

		Kmeans(int nNbComp, int nClassNumber, int nInputDimension);

		~Kmeans();

		/* This function carries out the k-means method training phase. From learning data
			(previously ramdomly generated pixels), we obtain a prototype of the classes.*/
		int train(HyperMat learningData, HyperMat proto, double threshold, int nbIter);

		/* This function carries out the k-means method clustering phase. From a classes prototype,
			it classifies all the images pixels in the different classes.*/
		int classify(HyperMat proto, HyperMat mask, double *weights, HyperMat images[NBIMAGES_MAX],
						HyperMat classes_image);


	private :

		/* This function classifies a peculiar pixel represented by its values in the different
			images. For that, it uses a classes prototype previously computed. */
		int classifyValue(HyperMat proto, double **val);


};

extern "C" {

#endif
	/** These functions can be accessed by C functions */

	/**@name kmeansInit
	 * @description This fonction is used to initialize an object that allows to access others functions
	 * @param nNbComp : number of component for each pixel
	 * @param nClassNumber : class number, which is the number of pixels clusters that we want
	 * 		to make from data images
	 * @param nInputDimension : dimension of the input space, which is the number of computed
	 *		images considered (original image, average image, standard deviation image, ...)
	 *
	 */
	void kmeansInit(int nNbComp, int nClassNumber, int nInputDimension);


	/**@name kmeansDeinit
	 * @description This fonction is used to desinitialize the object that allows to access others functions
	 *
	 */
	void kmeansDeinit();


	/**@name kmeansTrain
	 * @description This function carries out the k-means method training phase.
	 *		From learning data (previously ramdomly generated pixels), we obtain a prototype of the classes.
	 * @param learningData : ramdomly generated pixels, with their different values in the computed images.
	 *		This data is obtained with 'imlearningdata' Scilab function.
	 * @param proto : result prototype of the classes.
	 * @param threshold : value to stop training computation. Default value used is 1E-10
	 * @param nbIter : minimal number of iterations during training computation. Default value used is 10.
	 */
	int kmeansTrain(HyperMat learningData, HyperMat proto, double threshold, int nbIter);


	/**@name kmeansClassify
	 * @description This function carries out the k-means method clustering phase.
	 *		From a classes prototype, it classifies all the images pixels in the different classes.
	 * @param proto : prototype of the classes. This prototype is obtained with 'KmeansTrain' function.
	 * @param mask : mask of images. It defines what pixels are considered in clustering. Pixels that have a
	 *		value of 1 are considered, but not pixels with a value of 0. The mask has the same size as the 3
	 *		first dimensions of images.
	 * @param weights : relative importance of each computed image in clustering.
	 */
	int kmeansClassify(HyperMat proto, HyperMat mask, double *weights, HyperMat images[NBIMAGES_MAX],
						HyperMat classes_image );


#ifdef __cplusplus

}

#endif

#endif

