
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "kohonenlib.h"

#define _MAX(X, Y) ((X) > (Y)) ? (X) : (Y)
#define _MIN(X, Y) ((X) < (Y)) ? (X) : (Y)
#define ROUND(X) (floor(X+0.5))


// Kohonen SOM object
KohonenSOM *kohonen;


void kohonenInit(int nNbComp, int nClassNumber, int nInputDimension) {
	// Creation of KohonenSOM object
	kohonen = new KohonenSOM(nNbComp, nClassNumber, nInputDimension);
}

void kohonenDeinit() {
	// Destruction of KohonenSOM object
	if (kohonen)
		delete kohonen;
}


int kohonenTrain(HyperMat learningData, HyperMat proto, int nbIter) {
	// Kohonen SOM method training phase
	return kohonen->train(learningData, proto, nbIter);
}


//...................................................................................................


KohonenSOM::KohonenSOM(int nNbComp, int nClassNumber, int nInputDimension) :
				NeuralNet(nNbComp, nClassNumber, nInputDimension) {

}


KohonenSOM::~KohonenSOM() {

}


//################################################################################################################


int KohonenSOM::train(HyperMat learningData, HyperMat proto, int nbIter) {


	int nbPoints= learningData.dims[0];
	int nbComp = proto.dims[2];
	int i_fun, i_class, i_point, i_node, i_comp;
	int iWinner, iNeighbourHood, iLeft, iRight;
	double rError, rMin, rA, rB, rLearnRate;
	double *rDist;
	double rAux;

	if (!nbPoints) {
		return 1;
	}


	if (flags & GENERIC_NN_PROTO) {
		if (rDist = (double *) calloc (classCount, sizeof (double))) {

			// Prototype initialisation
			initProto(learningData, proto);


			i_point = 0;
			rA = (4.0 * classCount) / (3.0 * nbPoints);
			rB = 1;

			do {

				// Compute the distances between a point and the prototype, for each class
				for (i_class=0 ; i_class < classCount ; i_class++) {
					rDist[i_class] = 0;
					for (i_fun=0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
							rAux = LearningData(i_point % nbPoints, i_fun, i_comp) - Proto(i_fun, i_class, i_comp);
							rDist[i_class] += rAux * rAux;
						}
					}
					rDist[i_class] = sqrt (rDist[i_class]);
				}

				//searching the best match
				rMin = rDist[0];
				iWinner = 0;
				for (i_class=1 ; i_class < classCount ; i_class++) {
					if (rMin > rDist[i_class]) {
						rMin = rDist[i_class];
						iWinner = i_class;
					}
				}

				// Refining the weights
				// the neighbourhood variation is given by the law
				// iNeighbourHood = MAX(0, iClass - A * i_point)
				iNeighbourHood = _MAX(0, (int) ROUND (classCount - rA * i_point));
				iLeft = _MAX(0, iWinner - iNeighbourHood);
				iRight = _MIN(classCount - 1, iWinner + iNeighbourHood);
				rLearnRate = rB / sqrt (i_point + 1);

				for (i_node = iLeft ; i_node <= iRight ; i_node++) {
					rAux = rLearnRate * exp (- ((double)i_node - iWinner) * ((double)i_node - iWinner) /
											((double)iNeighbourHood * iNeighbourHood + 1.0));
					for (i_fun=0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
							PrivateProto(i_fun, i_node, i_comp) =
								Proto(i_fun, i_node, i_comp) +
								rAux * (LearningData(i_point % nbPoints, i_fun, i_comp) - Proto(i_fun, i_node, i_comp));
						}
					}
				}

				// Updating the weights
				rError = 0.0;
				for (i_class=0 ; i_class < classCount ; i_class++) {
					for (i_fun=0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
							rAux = Proto(i_fun, i_class, i_comp) - PrivateProto(i_fun, i_class, i_comp);
							rError += rAux * rAux;
							Proto(i_fun, i_class, i_comp) = PrivateProto(i_fun, i_class, i_comp);
						}
					}
				}
				rError = sqrt (rError);

				i_point++;

			} while ((i_point < 2*nbPoints) || (i_point < nbIter));

			free (rDist);
		}
	}

	// Sort prototype classes in increasing order
	sortProto(proto);

	return 0;
}




