/******************************************************************************/
/* Functions used to convert a 24-bit color image into a 8-bit color image    */
/* and to compute colormaps to display it with 4096 colors                    */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include "int_createcmap4096colors.h"

#define RMASK      0x7C00
#define RSHIFT          0
#define GMASK      0x7C00
#define GSHIFT          5
#define BMASK      0x7C00
#define BSHIFT         10	

/* RANGE forces a to have a value between b and c */
#define RANGE(a,b,c) { if (a < b) a = b;  if (a > c) a = c; }

/******************************************************************************/
/* Interface Scilab <-> createcmap4096colors                                  */
/******************************************************************************/
/* Creation : 06/03/2002 (V. COUVERT)                                         */
/* Last modification : 29/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_createcmap4096colors(char *fname)
{
	int l_redimage, m_redimage, n_redimage,
	    l_greenimage, m_greenimage, n_greenimage,
	    l_blueimage, m_blueimage, n_blueimage,
	    *l_image8, m_image8, n_image8,
	    *l_rmap, m_rmap, n_rmap,
	    *l_gmap, m_gmap, n_gmap,
	    *l_bmap, m_bmap, n_bmap;

	int ierr=0;
	    
	int minlhs=4, maxlhs=4, minrhs=3, maxrhs=3;
	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */
	
	GetRhsVar(1, "i", &m_redimage, &n_redimage, &l_redimage);
	GetRhsVar(2, "i", &m_greenimage, &n_greenimage, &l_greenimage);
	GetRhsVar(3, "i", &m_blueimage, &n_blueimage, &l_blueimage);

	createcmap4096colors(istk(l_redimage), istk(l_greenimage), istk(l_blueimage), m_redimage, n_redimage, &l_image8, &l_rmap, &l_gmap, &l_bmap, &n_rmap, &ierr);
	
	/* In case of error */
	if(ierr>0)
	{
		Scierror(999, "%s : Internal error \r\n",fname);
	}

	m_rmap=m_gmap=m_bmap=1;
	n_gmap=n_bmap=n_rmap;
	
	CreateVarFromPtr(4, "i", &m_redimage, &n_redimage, &l_image8);
	CreateVarFromPtr(5, "i", &m_rmap, &n_rmap, &l_rmap);
	CreateVarFromPtr(6, "i", &m_gmap, &n_gmap, &l_gmap);
	CreateVarFromPtr(7, "i", &m_bmap, &n_bmap, &l_bmap);
	
	LhsVar(1)=4;
	LhsVar(2)=5;
	LhsVar(3)=6;
	LhsVar(4)=7;
	
	if (l_image8) {
		free(l_image8);
	}
	if (l_rmap) {
		free(l_rmap);
	}
	if (l_gmap) {
		free(l_gmap);
	}
	if (l_bmap) {
		free(l_bmap);
	}

	
	
	return(0);
}


/******************************************************************************/
/* Conversion function                                                        */
/******************************************************************************/
int createcmap4096colors(int *redimage, int *greenimage, int *blueimage, 
						 int w, int h, int **p8, int **rmap, int **gmap, int **bmap,
						 int *cmap_size, int *ierr)
{
	int *pp, *thisline, *nextline, *thisptr, *nextptr, *tmpptr;
	int  r1, g1, b1, i, j, val, imax, jmax, nc=4096, x, y, line_count=0, row_count=0;

	/* Memory allocation */
	if (!(*rmap = (int*)calloc(1, sizeof(int)*nc)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*gmap = (int*)calloc(1, sizeof(int)*nc)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*bmap = (int*)calloc(1, sizeof(int)*nc)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*p8= (int*)calloc(1, sizeof(int)*w*h)))
	{
		*ierr=1;
		return(-1);
    	}
	    	
	pp=*p8;
	imax = h-1;
	jmax = w-1;

  	/* Colormap creation*/
  	for(i=0; i<nc; i++)
  	{
    		(*rmap)[i] = (((i<<RSHIFT) & RMASK) * (nc-1) + RMASK/2) / RMASK;
    		(*gmap)[i] = (((i<<GSHIFT) & GMASK) * (nc-1) + GMASK/2) / GMASK;
    		(*bmap)[i] = (((i<<BSHIFT) & BMASK) * (nc-1) + BMASK/2) / BMASK;
  	}
  
  	/* New 8-bit image creation */
  	thisline = (int *) malloc(3*w*sizeof(int));
  	nextline = (int *) malloc(3*w*sizeof(int));
  	if(!thisline || !nextline)
  	{
    		if(thisline)
    			free(thisline);
    		if(nextline)
    			free(nextline);
  	}
  
  	/* First image line */
  	row_count=0;
	line_count=0;
	for(j=w,tmpptr=nextline;j;j--)
	{
		*tmpptr++=redimage[row_count+w*line_count];
		*tmpptr++=greenimage[row_count+w*line_count];
		*tmpptr++=blueimage[row_count+w*line_count];
		row_count++;
	}

	for(i=0; i<h; i++)
  	{
    		tmpptr = thisline;  thisline = nextline;  nextline = tmpptr;   

	    	if(i!=imax)   /* Next image line */
      		{
    			for(j=w,tmpptr=nextline;j;j--)
			{
				*tmpptr++=redimage[row_count+w*line_count];
				*tmpptr++=greenimage[row_count+w*line_count];
				*tmpptr++=blueimage[row_count+w*line_count];
				row_count++;
			}
		}

    		for (j=0, thisptr=thisline, nextptr=nextline; j<w; j++,pp++)
    		{
      			r1 = (*thisptr++) <<4;
      			g1 = (*thisptr++) <<4;
      			b1 = (*thisptr++) <<4;
      			RANGE(r1,0,(nc-1));
      			RANGE(g1,0,(nc-1));
      			RANGE(b1,0,(nc-1));  
      
      			/* Current pixel value */
      			val = (((r1&RMASK)>>RSHIFT) | ((g1&GMASK)>>GSHIFT) | ((b1&BMASK)>>BSHIFT));
      			*pp = val;
	      		/* Errors computing */
      			r1 -= (*rmap)[val];
      			g1 -= (*gmap)[val];
      			b1 -= (*bmap)[val];
      
      			/* Add errors to neighbor pixels */
      			if (j!=jmax)
      			{  
				thisptr[0] += (r1*7) / 64;
				thisptr[1] += (g1*7) / 64;
				thisptr[2] += (b1*7) / 64;
      			}
      
      			if (i!=imax)
      			{	
				nextptr[0] += (r1*5) / 64;
				nextptr[1] += (g1*5) / 64;
				nextptr[2] += (b1*5) / 64;

				if (j>0)
				{  
	  				nextptr[-3] += (r1*3) / 64;
	  				nextptr[-2] += (g1*3) / 64;
	  				nextptr[-1] += (b1*3) / 64;
				}

				if (j!=jmax)
				{  
	  				nextptr[3] += (r1) / 64;
	  				nextptr[4] += (g1) / 64;
	  				nextptr[5] += (b1) / 64;
				}
				nextptr += 3;
      			}
    		}
  	}
  	free(thisline);
  	free(nextline);
  	
  	*cmap_size=nc;

	*ierr=0;
	return(0);

}
