/******************************************************************************/
/* Functions used to close holes in a grey level image from Scilab            */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include "int_imclosehole.h"

/******************************************************************************/
/* Interface Scilab <-> imclosehole                                           */
/******************************************************************************/
/* Creation : 10/04/2002 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imclosehole(char *fname)
{
	int l_image, m_image, n_image;
	double *l_closed_image;

	int ierr=0;
	    
	int minlhs=1, maxlhs=1, minrhs=1, maxrhs=1;
	
	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Ouput arguments 
	
	GetRhsVar(1, "d", &m_image, &n_image, &l_image);

	close_hole_image(stk(l_image), m_image, n_image, &l_closed_image, &ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(3,"d", &m_image, &n_image, &l_closed_image);

	if (l_closed_image) {
		free(l_closed_image);
	}

	LhsVar(1)=3;

	return(0);
}

/******************************************************************************/
/* Closing function                                                           */
/******************************************************************************/
/* Creation : 10/04/2002 (V. COUVERT)                                         */
/* Last modification : 02/07/2004  (V. POMMEREUL)                             */
/******************************************************************************/
int close_hole_image(double *image, int width, int height, 
					 double **closed_image, int *ierr)
{
	int x, y, nb_occur, max_occur=0, i=0, j=0;
	double cornervalue[4], backvalue=10;

	/* Searching for background value */
	cornervalue[0]=image[0];
	cornervalue[1]=image[width-1];
	cornervalue[2]=image[(height-1)*width];
	cornervalue[3]=image[width-1+(height-1)*width];

	for(i=0 ; i<4 ; i++)
	{
		nb_occur=0;
		for(j=0 ; j<4 ; j++)
		{
			if(cornervalue[j]==cornervalue[i])
				nb_occur++;
		}
		if(nb_occur > max_occur) {
			backvalue = cornervalue[i];
			max_occur = nb_occur;
		}
	}

	/* Memory allocation */
	if (!(*closed_image = (double*)calloc(width*height, sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}

	/* Put background = -1 by scanning lines and columns */
	/* To avoid stack problems if many points in background */
	for(y=0;y<height;y++)
	{
		for(x=0; image[x+y*width] ==backvalue && x<width;x++)
		{
			(*closed_image)[x+y*width]=1;
		}
		for(x=width-1 ; image[x+y*width]==backvalue && x>0 ; x--)
		{
			(*closed_image)[x+y*width]=1;
		}
	}
	for(x=0;x<width;x++)
	{
		for(y=0;image[x+y*width] ==backvalue && y<height;y++)
		{
			(*closed_image)[x+y*width]=1;
		}
		for(y=height-1;image[x+y*width] ==backvalue && y>0;y--)
		{
			(*closed_image)[x+y*width]=1;
		}
	}
	/* End of lines/column scanning */
	/* To finish background "labelling", we use a recursive function */
	for(y=0;y<height;y++)
	{
		for(x=0;x<width;x++)
		{
			if( (*closed_image)[x+y*width]==1 )
			{
				background_neighbor(image,closed_image,x,y,backvalue,width,height,8);
			}
		}
	}
	/* End of background "labelling" */

	/* WARNING : this function return the negative image of what we want */
	/* What we want is background=0 and objects=1 */
	/* Inversion is done in imclosehole.sci */
	*ierr=0;
	return(0);
}

/******************************************************************************/
/* Recursive function to detect 4-connexity or 8-connexity in background      */
/******************************************************************************/
/* Creation : 26/04/2002 (V. COUVERT)                                         */
/* Last modification : 22/05/2002 (V. COUVERT)                                */
/******************************************************************************/
void background_neighbor(double *image, double **closed_image, int x, int y, double back,
 int w, int h, int connectivity)
{
	if((connectivity == 4) || (connectivity == 8))
	{
		if(x<w-1)
		{
			if( image[x+1+y*w] == back && (*closed_image)[x+1+y*w]==0 )
			{
				(*closed_image)[x+1+y*w]=1;
				background_neighbor(image, closed_image, x+1, y, back, w, h, connectivity);
			}
		}
		if(x>0)
		{
			if(image[x-1+y*w]== back && (*closed_image)[x-1+y*w]==0 )
			{
				(*closed_image)[x-1+y*w]=1;
				background_neighbor(image, closed_image, x-1, y, back, w, h, connectivity);
			}
		}
		if(y<h-1)
		{
			if(image[x+(y+1)*w] ==back && (*closed_image)[x+(y+1)*w]==0 )
			{
				(*closed_image)[x+(y+1)*w]=1;
				background_neighbor(image,closed_image,x,y+1,back,w,h,connectivity);
			}
		}
		if(y>0)
		{
			if(image[x+(y-1)*w]==back && (*closed_image)[x+(y-1)*w]==0 )
			{
				(*closed_image)[x+(y-1)*w]=1;
				background_neighbor(image,closed_image,x,y-1,back,w,h,connectivity);
			}
		}
	}

	if(connectivity==8)
	{
		if(y<h-1 && x<w-1)
		{
			if(*(image+x+1+(y+1)*w)==back && (*closed_image)[x+1+(y+1)*w]==0 )
			{
				(*closed_image)[x+1+(y+1)*w]=1;
				background_neighbor(image,closed_image,x+1,y+1,back,w,h,connectivity);
			}
		}
		if(y<h-1 && x>0)
		{
			if(*(image+x-1+(y+1)*w)==back && (*closed_image)[x-1+(y+1)*w]==0 )
			{
				(*closed_image)[x-1+(y+1)*w]=1;
				background_neighbor(image,closed_image,x-1,y+1,back,w,h,connectivity);
			}
		}
		if(x>0 && y>0)
		{
			if(*(image+x-1+(y-1)*w)==back && (*closed_image)[x-1+(y-1)*w]==0)
			{
				(*closed_image)[x-1+(y-1)*w]=1;
				background_neighbor(image,closed_image,x-1,y-1,back,w,h,connectivity);
			}
		}
		if(y>0 && x<w-1)
		{
			if(*(image+x+1+(y-1)*w)==back && (*closed_image)[x+1+(y-1)*w]==0)
			{
				(*closed_image)[x+1+(y-1)*w]=1;
				background_neighbor(image,closed_image,x+1,y-1,back,w,h,connectivity);
			}
		}
	}
}

