/******************************************************************************/
/* Functions used to read elements in a dicom file header from Scilab         */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include <math.h>
#include <string.h>

#include "int_imdcmelement.h"

/* CTN library header files */
#include "dicom.h"
#include "condition.h"
#include "lst.h"
#include "dicom_objects.h"
#include "dicom_uids.h"
#include "ctnthread.h"

char * substring(char *string, int position)
{
  char *str = string; int i;
  for (i=1; i<position; i++)
    if ((str = strchr(str, '\\'))) str++;
  return str;
}

/******************************************************************************/
/* Interface Scilab <-> imdcmelement                                          */
/******************************************************************************/
/* Creation : 13/06/2002 (V. COUVERT)                                         */
/* Last modification : 23/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imdcmelement(char *fname) {

	int l_filename, m_filename, n_filename,
	    l_parameters, m_parameters, n_parameters,
	    l_parameters_nb, m_parameters_nb, n_parameters_nb,
	    m_results, n_results;
	double *l_results;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Output arguments 

	GetRhsVar(1, "c", &m_filename, &n_filename, &l_filename);

	GetRhsVar(2, "i", &m_parameters, &n_parameters, &l_parameters);

	GetRhsVar(3, "i", &m_parameters_nb, &n_parameters_nb, &l_parameters_nb);

	dcm_element(cstk(l_filename), istk(l_parameters), *istk(l_parameters_nb), &l_results, &ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}
	m_results=1;
	n_results=*istk(l_parameters_nb);

	CreateVarFromPtr(4,"d", &m_results, &n_results, &l_results);

	if (l_results) {
		free(l_results);
	}

	LhsVar(1)=4;

	return(0);
}

/******************************************************************************/
/* Header file reading function                                               */
/******************************************************************************/
/* Creation : 13/06/2001 (V. COUVERT)                                         */
/* Last modification : 09/06/2005 (M. TALLUR)                                 */
/******************************************************************************/
int dcm_element(char *filename, int *parameters, int parameters_nb, 
				double **results, int *ierr)
{
	DCM_OBJECT *dcmObject = NULL;
	DCM_ELEMENT dcmElement;
	CONDITION cond;
	unsigned long nOptions = DCM_ORDERLITTLEENDIAN | DCM_PART10FILE;
	U32 rtnSize;
	void *ctx=NULL;
	int i;
	unsigned char *tampon;

	// Memory allocation
	if (!(*results = (double*)calloc(parameters_nb, sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}
	if (!(tampon = (char*)malloc(sizeof(char)))) {
		*ierr=1;
		return 1;
	}

	// Openning the file 
	cond = DCM_OpenFile (filename, nOptions, &dcmObject);

	if(cond!=DCM_NORMAL) /* Opening file failed : maybe it is not a PART10 file... */
	{
		nOptions &= ~(DCM_PART10FILE);
		cond = DCM_OpenFile (filename, nOptions, &dcmObject);
		if(cond!=DCM_NORMAL) /* Opening file has failed : try with BIGENDIANORDER */
		{
			nOptions |= DCM_PART10FILE;
			nOptions &= ~(DCM_ORDERMASK);
			nOptions |= DCM_ORDERBIGENDIAN;
			cond = DCM_OpenFile (filename, nOptions, &dcmObject);
			if(cond!=DCM_NORMAL) /* Opening file failed : maybe it is not a PART10 file... */
			{
				nOptions &= ~(DCM_PART10FILE);
				cond = DCM_OpenFile (filename, nOptions, &dcmObject);
				if(cond!=DCM_NORMAL)
				{
					sciprint("File %s doesn't exist or read access denied.\n", filename);
					
					for(i=0 ; i<parameters_nb ; i++)
						*results[i]=-1.0;
					
					return(0);
				}
			}
		}
	}

	// Slice thickness 
	if(parameters[0]==1)
	{
		dcmElement.tag=DCM_ACQSLICETHICKNESS;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[1]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Inversion time 
	if(parameters[1]==1)
	{
		dcmElement.tag=DCM_ACQINVERSIONTIME;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		// indeed, strncpy does not append the terminating character 
		(*results)[parameters[3]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Echo time 
	if(parameters[4]==1)
	{
		dcmElement.tag=DCM_ACQECHOTIME;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[5]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Repetion time
	if(parameters[6]==1)
	{
		dcmElement.tag=DCM_ACQREPETITIONTIME;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[7]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Number of averages 
	if(parameters[8]==1)
	{
		dcmElement.tag=DCM_ACQNUMBEROFAVERAGES;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[9]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Imaging frequency 
	if(parameters[10]==1)
	{
		dcmElement.tag=DCM_ACQIMAGINGFREQUENCY;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[11]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Matrix size 
	if(parameters[12]==1)
	{
		dcmElement.tag=DCM_SHAQACQUISITIONCOLUMNS;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[13]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Field of view 
	if(parameters[14]==1)
	{
		dcmElement.tag=DCM_SHRELFOV;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[15]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Total receiver gain */
	if(parameters[16]==1)
	{
		dcmElement.tag=DCM_SHAQRECEIVERTOTALGAIN;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[17]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Reconstruction scale factor */
	if(parameters[18]==1)
	{
		dcmElement.tag=DCM_SHAQRECONSTRUCTIONSCALEFACTOR;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[19]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Transmitter calibration */
	if(parameters[20]==1)
	{
		dcmElement.tag=DCM_SHAQTRANSMITTERCALIBRATION;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[21]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Study date */
	if(parameters[22]==1)
	{
		dcmElement.tag=DCM_IDSTUDYDATE;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[23]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Serial number */
	if(parameters[24]==1)
	{
		dcmElement.tag=DCM_RELSERIESNUMBER;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[25]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Image number */
	if(parameters[26]==1)
	{
		dcmElement.tag=DCM_RELIMAGENUMBER;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[27]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Real acquisition time */
	if(parameters[28]==1)
	{
		dcmElement.tag=DCM_IDACQUISITIONTIME;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[29]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Real image time */
	if(parameters[30]==1)
	{
		dcmElement.tag=DCM_IDIMAGETIME;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[31]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* Slice location */
	if(parameters[32]==1)
	{
		dcmElement.tag=DCM_RELSLICELOCATION;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[33]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Field of view (height in mm) 
	if(parameters[34]==1)
	{
		dcmElement.tag=DCM_SHRELFOV;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[35]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	// Field of view (width in mm)
	if(parameters[36]==1)
	{
		dcmElement.tag=DCM_SHRELFOV;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		   (*results)[parameters[37]-1]=(double)atof(substring(tampon, 2));
		ctx=NULL;
	}

	/* SHREL Image position vector (sag) */
	if(parameters[38]==1)
	{
		dcmElement.tag=DCM_SHRELIMAGEPOSITION;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[39]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* SHREL Image position vector (cor) */
	if(parameters[40]==1)
	{
		dcmElement.tag=DCM_SHRELIMAGEPOSITION;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[41]-1]=(double)atof(substring(tampon, 2));
		ctx=NULL;
	}

	/* SHREL Image position vector (tra) */
	if(parameters[42]==1)
	{
		dcmElement.tag=DCM_SHRELIMAGEPOSITION;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[43]-1]=(double)atof(substring(tampon, 3));
		ctx=NULL;
	}

	/* SHREL Image normal cosinus dir. (sag) */
	if(parameters[44]==1)
	{
		dcmElement.tag=DCM_SHRELIMAGENORMAL;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[45]-1]=(double)atof(tampon);
		ctx=NULL;
	}

	/* SHREL Image normal cosinus dir. (cor) */
	if(parameters[46]==1)
	{
		dcmElement.tag=DCM_SHRELIMAGENORMAL;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[47]-1]=(double)atof(substring(tampon, 2));
		ctx=NULL;
	}

	/* SHREL Image normal cosinus dir. (tra) */
	if(parameters[48]==1)
	{
		dcmElement.tag=DCM_SHRELIMAGENORMAL;
		DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
		dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
		DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
		tampon = (char *)realloc(tampon,(dcmElement.length+1)*sizeof(char));
		strncpy(tampon, dcmElement.d.ot, dcmElement.length);
		tampon[dcmElement.length] = '\0';
		(*results)[parameters[49]-1]=(double)atof(substring(tampon, 3));
		ctx=NULL;
	}

	/* Closing file */
	DCM_CloseObject(&dcmObject);

	free(tampon);


	*ierr=0;
	return(0);
}

/******************************************************************************/
/* Contents of *parameters variable                                           */
/******************************************************************************/
/* parameters[0] : read 'slice thickness' or not                            */
/* parameters[1] : position of 'slice thickness' in call                    */
/* parameters[2] : read 'inversion time' or not                             */
/* parameters[3] : position of 'inversion time' in call                     */
/* parameters[4] : read 'echo time' or not                                  */
/* parameters[5] : position of 'echo time' in call                          */
/* parameters[6] : read 'repetition time' or not                            */
/* parameters[7] : position of 'repetition time' in call                    */
/* parameters[8] : read 'number of averages' or not                         */
/* parameters[9] : position of 'number of averages' in call                 */
/* parameters[10] : read 'imaging frequency' or not                         */
/* parameters[11] : position of 'imaging frequency' in call                 */
/* parameters[12] : read 'matrix size' or not                               */
/* parameters[13] : position of 'matrix size' in call                       */
/* parameters[14] : read 'field of view' or not                             */
/* parameters[15] : position of 'field of view' in call                     */
/* parameters[16] : read 'total receiver gain' or not                       */
/* parameters[17] : position of 'total receiver gain' in call               */
/* parameters[18] : read 'reconstruction scale factor' or not               */
/* parameters[19] : position of 'reconstruction scale factor' in call       */
/* parameters[20] : read 'transmitter calibration' or not                   */
/* parameters[21] : position of 'transmitter calibration' in call           */
/* parameters[22] : read 'study date' or not                                */
/* parameters[23] : position of 'study date' in call                        */
/* parameters[24] : read 'serial number' or not                             */
/* parameters[25] : position of 'serial number' in call                     */
/* parameters[26] : read 'image number' or not                              */
/* parameters[27] : position of 'image number' in call                      */
/* parameters[28] : read 'real acquisition time' or not                     */
/* parameters[29] : position of 'real acquisition time' in call             */
/* parameters[30] : read 'real image time' or not                           */
/* parameters[31] : position of 'real image time' in call                   */
/* parameters[32] : read 'slice location' or not                            */
/* parameters[33] : position of 'slice location' in call                    */
/* parameters[34] : read 'field of view h' or not                           */
/* parameters[35] : position of 'field of view h' in call                   */
/* parameters[36] : read 'field of view w' or not                           */
/* parameters[37] : position of 'field of view w' in call                   */
/* parameters[38] : read 'image position vector s' or not                   */
/* parameters[39] : position of 'image position vector s' in call           */
/* parameters[40] : read 'image position vector c' or not                   */
/* parameters[41] : position of 'image position vector c' in call           */
/* parameters[42] : read 'image position vector t' or not                   */
/* parameters[43] : position of 'image position vector t' in call           */
/* parameters[44] : read 'image normal cosinus s' or not                    */
/* parameters[45] : position of 'image normal cosinus s' in call            */
/* parameters[46] : read 'image normal cosinus c' or not                    */
/* parameters[47] : position of 'image normal cosinus c' in call            */
/* parameters[48] : read 'image normal cosinus t' or not                    */
/* parameters[49] : position of 'image normal cosinus t' in call            */
/******************************************************************************/
