/******************************************************************************/
/* Functions used to get edges of an image from Scilab                        */
/* Using DERICHE method                                                       */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "stack-c.h"
#include "int_imderiche.h"

/******************************************************************************/
/* Interface Scilab <-> imderiche                                             */
/******************************************************************************/
/* Creation : 10/04/2002 (V. COUVERT)                                         */
/* Last modification : 20/10/04 (CC, PL)                                      */
/******************************************************************************/
int int_imderiche(char *fname)
{
	int l_image, m_image, n_image,
	    l_alpha, m_alpha, n_alpha,
	    l_dist, m_dist, n_dist,
	    m_filtered_image, n_filtered_image;
	double *l_filtered_image;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Ouput arguments */

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_alpha, &n_alpha, &l_alpha);
	GetRhsVar(3, "d", &m_dist, &n_dist, &l_dist);

	deriche(stk(l_image),m_image,n_image,*stk(l_alpha),*stk(l_dist),&l_filtered_image,&ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(4,"d", &m_image, &n_image, &l_filtered_image);

	if (l_filtered_image) {
		free(l_filtered_image);
	}

	LhsVar(1)=4;

	return(0);
}


double N(double *Ix, double *Iy, int width, int x, int y)
{
	return(sqrt(Ix[x+y*width]*Ix[x+y*width] + Iy[x+y*width]*Iy[x+y*width]));

}


/******************************************************************************/
/* filtered image computing                                                   */
/******************************************************************************/
int deriche(double *image, int width, int height, double alpha, double dist, double **filtered_image, int *ierr)
{
	int x, y, xp, yp, mgx=2, mgy=2, temp;
	double *Yp, *Ym, *tmp, *Iy, *image_transposed, *Ix, *Ix_transposed;
	double k, a, a0, a1, a2, a3, b1, b2;
	int Sx, Sy;
	double dx, dy, lambda;
	double r, c, dr, dc;
	double N0, N1, N00, N01, N10, N11;

	/* Memory allocation */
	if (!(Ix_transposed = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(Ix = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*filtered_image = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(Yp = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(Ym = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(tmp = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(Iy = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(image_transposed = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}

	/* Filter computation */
	/* Iy = x smoothing and then y derivation */

	/* Smoothing*/
	k = (1-exp(-alpha))*(1-exp(-alpha))/(1+2*alpha*exp(-alpha)-exp(-2*alpha));
	a0 = k;
	a1 = k*(alpha-1)*exp(-alpha);
	a2 = k*(alpha+1)*exp(-alpha);
	a3 = -k*exp(-2*alpha);
	b1 = -2*exp(-alpha);
	b2 = exp(-2*alpha);
	Sx = width - 2*mgx;
	Sy = height- 2*mgy;

	for(y=mgy;y<Sy+mgy;y++)
	{
		/* Yp filtering */
		for(x=mgx;x<Sx+mgx;x++)
		{
			Yp[x+y*width] = a0*image[x+y*width] + a1*image[x-1+y*width] - b1*Yp[x-1+y*width] - b2*Yp[x-2+y*width];
		}
		/* Ym filtrage */
		for(x=Sx+mgx-1;x>=mgx;x--)
		{
			Ym[x+y*width]=a2*image[x+1+y*width] + a3*image[x+2+y*width] - b1*Ym[x+1+y*width] - b2*Ym[x+2+y*width];
		}
		/* Compute Y */
		for(x=mgx;x<Sx+mgx;x++)
		{
			tmp[x+y*width] = Yp[x+y*width] + Ym[x+y*width];
		}
	}

	/* Derivation */
	k = exp(-alpha);
	a = (1-k)*(1-k);
	b1 = -2*k;
	b2 = k*k;
	Sx = width - 2*mgx;
	Sy = height- 2*mgy;

	/* Recursive filtering */
	for(x=mgx;x<Sx+mgx;x++)
	{
		/* Yp filtering */
		for(y= mgy;y<Sy+mgy;y++)
		{
			Yp[x+y*width] = a*tmp[x+(y-1)*width] - b1*Yp[x+(y-1)*width] - b2*Yp[x+(y-2)*width];
		}
		/* Ym filtering */
		for(y=Sy+mgy-1;y>=mgy;y--)
		{
			Ym[x+y*width] = -a*tmp[x+(y+1)*width] -  b1*Ym[x+(y+1)*width] - b2*Ym[x+(y+2)*width];
		}
		/* Compute Y */
		for(y=mgy;y<Sy+mgy;y++)
		{
			Iy[x+y*width] = Yp[x+y*width] + Ym[x+y*width];
		}
	}

	/* Ix = y smoothing and then x derivation*/

	/* Transposition */
	for(y=0;y<height;y++)
	{
		for(x=0;x<width;x++)
		{
			image_transposed[y+x*height] = image[x+y*width];
		}
	}
	temp=width;
	width=height;
	height=temp;

	/* Smoothing */
	k = (1-exp(-alpha))*(1-exp(-alpha))/(1+2*alpha*exp(-alpha)-exp(-2*alpha));
	a0 = k;
	a1 = k*(alpha-1)*exp(-alpha);
	a2 = k*(alpha+1)*exp(-alpha);
	a3 = -k*exp(-2*alpha);
	b1 = -2*exp(-alpha);
	b2 = exp(-2*alpha);
	Sx = width - 2*mgx;
	Sy = height- 2*mgy;

	for(y=mgy;y<Sy+mgy;y++)
	{
		/* Yp filtering */
		for(x=mgx;x<Sx+mgx;x++)
		{
			Yp[x+y*width] = a0*image_transposed[x+y*width] + a1*image_transposed[x-1+y*width] - b1*Yp[x-1+y*width] - b2*Yp[x-2+y*width];
		}
		/* Ym filtrage */
		for(x=Sx+mgx-1;x>=mgx;x--)
		{
			Ym[x+y*width]=a2*image_transposed[x+1+y*width] + a3*image_transposed[x+2+y*width] - b1*Ym[x+1+y*width] - b2*Ym[x+2+y*width];
		}
		/* Compute Y */
		for(x=mgx;x<Sx+mgx;x++)
		{
			tmp[x+y*width] = Yp[x+y*width] + Ym[x+y*width];
		}
	}

	/* Derivation */
	k = exp(-alpha);
	a = (1-k)*(1-k);
	b1 = -2*k;
	b2 = k*k;
	Sx = width - 2*mgx;
	Sy = height- 2*mgy;

	/* Recursive filtering */
	for(x=mgx;x<Sx+mgx;x++)
	{
		/* Yp filtering */
		for(y= mgy;y<Sy+mgy;y++)
		{
			Yp[x+y*width] = a*tmp[x+(y-1)*width] - b1*Yp[x+(y-1)*width] - b2*Yp[x+(y-2)*width];
		}
		/* Ym filtering */
		for(y=Sy+mgy-1;y>=mgy;y--)
		{
			Ym[x+y*width] = -a*tmp[x+(y+1)*width] -  b1*Ym[x+(y+1)*width] - b2*Ym[x+(y+2)*width];
		}
		/* Compute Y */
		for(y=mgy;y<Sy+mgy;y++)
		{
			Ix_transposed[x+y*width] = Yp[x+y*width] + Ym[x+y*width];
		}
	}

	/* Transposition */
	for(y=0;y<height;y++)
	{
		for(x=0;x<width;x++)
		{
			Ix[y+x*height] = Ix_transposed[x+y*width];
		}
	}
	temp=width;
	width=height;
	height=temp;


	/* Searching for extrema */
	Sx = width - 2*mgx;
	Sy = height- 2*mgy;


	for(y=mgy+(int)dist;y<=Sy+mgy-(int)dist-1;y++)
	{
		for(x=mgx+(int)dist;x<=Sx+mgx-(int)dist-1;x++)
		{
			N0 = N(Ix,Iy,width,x,y);
			(*filtered_image)[x+y*width] = N0;

			if(N0>0.1)
			{
				dx = Ix[x+y*width]/N0;
				dy = Iy[x+y*width]/N0;

				for(lambda=-dist;lambda<=dist;lambda+=0.25)
				{
				        c = x+lambda*dx;
					r = y+lambda*dy;
					xp=(int)(c);
					yp=(int)(r);
					dc = c-xp;
                	                dr = r-yp;
					
					N00 = N(Ix,Iy,width,xp,yp);
					N01 = N(Ix,Iy,width,xp,yp+1);
					N10 = N(Ix,Iy,width,xp+1,yp);
					N11 = N(Ix,Iy,width,xp+1,yp+1);
					N1 = N00+dr*(N01-N00)+dc*(N10-N00)+dr*dc*(N00-N01-N10+N11);
				
					/* si on trouve au moins un point qui a un gradient plus fort dans le voisinage on met le resultat  0 et on arrete */
					if (N1-N0>1e-6)
					{
						(*filtered_image)[x+y*width] = 0;
						lambda=2*dist;
					}
				}
			}
		}
	}


	/* Memory freeing */
	free(tmp);
	free(Ix);
	free(Iy);
	free(image_transposed);
	free(Ix_transposed);
	free(Yp);
	free(Ym);

	*ierr=0;
	return(0);
}
