
/******************************************************************************/
/* Fonctions used to carry out the fuzzy k-means method classification phase  */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include <fuzzykmeanslib.h>
#include "int_imfuzzykmeansclassify.h"

/******************************************************************************/
/* Interface Scilab<->imfuzzykmeansclassify                                   */
/******************************************************************************/
/* Creation : 01/04/2004 (V. POMMEREUL)                                       */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/

int int_imfuzzykmeansclassify(char *fname) {

	// Input and output arguments
	static int l_dimension, m_dimension, n_dimension,
			   l_prototype, m_prototype, n_prototype,
			   l_mask, m_mask, n_mask,
			   l_weights, m_weights, n_weights,
			   l_fuzzyness, m_fuzzyness, n_fuzzyness,
			   m_image, n_image,
			   m_classes_image, n_classes_image;
	static int l_images[NBIMAGES_MAX];
	double *l_classes_image;
	HyperMat h_proto, h_mask, h_images[NBIMAGES_MAX], h_classes_image;

	double threshold1 = -1.0;
	double threshold2 = 0.05;

	// Number of input and output arguments
	int minlhs = 1, maxlhs = 1;
	int minrhs = 6, maxrhs = 15;

	int ierr=0;

	int ind_var=1;
	int ind_res=1;

	int dim, nbIm;
	double *s_mask, *s_images[NBIMAGES_MAX], *proto, *weights, fuzzyness;

	int i;

	// Verify number of arguments
	CheckRhs(minrhs, maxrhs); // Input arguments
	CheckLhs(minlhs, maxlhs); // Output arguments

	// Get right hand side arguments

	// Get dimension of image
	GetRhsVar(ind_var++, "i", &m_dimension, &n_dimension, &l_dimension);
	dim = *istk(l_dimension);

	// Get classes prototype
	if (dim == 2 | dim == 3) {
		// The prototype is a matrix
		GetRhsVar(ind_var++, "d", &m_prototype, &n_prototype, &l_prototype);
		proto = stk(l_prototype);
		nbIm = m_prototype;

	} else {
		// The prototype is an hypermatrix
		GetRhsHMat(ind_var++, &h_proto);
		nbIm = h_proto.dims[0];
	}

	// Get the image mask
	if (dim == 2) {
		// The mask is a matrix
		GetRhsVar(ind_var++, "d", &m_mask, &n_mask, &l_mask);
	} else {
		// The mask is an hypermatrix
		GetRhsHMat(ind_var++, &h_mask);
	}

	// Get the weights associated to each image
	GetRhsVar(ind_var++, "d", &m_weights, &n_weights, &l_weights);
	weights = stk(l_weights);

	// Get the fuzzyness
	GetRhsVar(ind_var++, "d", &m_fuzzyness, &n_fuzzyness, &l_fuzzyness);
	fuzzyness = *stk(l_fuzzyness);

	// Get data images
	for (i=0 ; i<nbIm ; i++) {
		if (dim == 2) {
			// The images are matrix
			GetRhsVar(ind_var++, "d", &m_image, &n_image, &l_images[i]);
		} else {
			// The images are hypermatrix
			GetRhsHMat(ind_var++, &h_images[i]);
		}
	}

	// Compute fuzzy k-means method classification phase
	if (dim == 2) {
		s_mask = stk(l_mask);

		for (i=0 ; i<nbIm ; i++) {
			s_images[i] = stk(l_images[i]);
		}
		// 2D images
		ierr = classify_fuzzykmeans_2D(proto, m_prototype, n_prototype, s_mask, weights, fuzzyness, threshold1,
									threshold2, s_images, m_image, n_image, &l_classes_image);

	} else if (dim == 3){
		// 3D images
		ierr = classify_fuzzykmeans_3D(proto, m_prototype, n_prototype, h_mask, weights, fuzzyness, threshold1,
									threshold2, h_images, &h_classes_image);
	} else {
		// 4D images
		ierr = classify_fuzzykmeans_4D(h_proto, h_mask, weights, fuzzyness, threshold1, threshold2, h_images, 												&h_classes_image);
	}


	// In case of error
	if(ierr!=0) {
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	// Return classes image
	if (dim == 2) {
		CreateVarFromPtr(ind_var, "d", &m_image, &n_image, &l_classes_image);
		if(l_classes_image) {
			free(l_classes_image);
		}
		LhsVar(ind_res++)=ind_var;

	} else {

		CreateHMat(ind_var,&h_classes_image);
		LhsVar(ind_res++) = ind_var;
		// Memory desallocation
		if (h_classes_image.dims) {
			free(h_classes_image.dims);
		}
		if (h_classes_image.R) {
			free(h_classes_image.R);
		}
	}

	return(0);

}


/*********************************************************************************************************************/
/*   Fuzzy k-means method classification phase function for a 2D image                                               */
/*********************************************************************************************************************/

int classify_fuzzykmeans_2D(double *proto, int nbIm, int nbClasses, double *mask, double *weights, double fuzzyness, 		double threshold1, double threshold2, double *images[NBIMAGES_MAX], int height, int width, double **classes_image) {

	int k;
	int err = 0;
	const int nbDim = 3;
	const int nbComp = 1;
	int dimvar = 0;
	HyperMat hproto, hmask, hclim, himages[NBIMAGES_MAX];

	// ........................................................................
	// Create an hypermatrix array for images
	for (k=0 ; k<nbIm ; k++) {
		himages[k].dimsize = nbDim;
		dimvar = 0;
		if (himages[k].dims = (int *) calloc(himages[k].dimsize, sizeof(int))) {
			himages[k].dims[dimvar++] = height;
			himages[k].dims[dimvar++] = width;
			himages[k].dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		himages[k].size = height * width * nbComp;
		himages[k].R = images[k];
		himages[k].it = 0;
	}


	// ........................................................................
	// Create an hypermatrix for the protoype
	hproto.dimsize = nbDim;
	dimvar = 0;
	if (hproto.dims = (int *) calloc(hproto.dimsize, sizeof(int))) {
		hproto.dims[dimvar++] = nbIm;
		hproto.dims[dimvar++] = nbClasses;
		hproto.dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	hproto.size = nbIm * nbClasses * nbComp;
	hproto.R = proto;
	hproto.it = 0;

	// ........................................................................
	// Create an hypermatrix for the mask
	hmask.dimsize = nbDim;
	dimvar = 0;
	if (hmask.dims = (int *) calloc(hmask.dimsize, sizeof(int))) {
		hmask.dims[dimvar++] = height;
		hmask.dims[dimvar++] = width;
		hmask.dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	hmask.size = height * width * nbComp;
	hmask.R = mask;
	hmask.it = 0;

	// ........................................................................
	// Memory allocation for the classes image
	if (*classes_image = (double*) calloc(height * width * nbComp, sizeof(double))) {
		// Create an hypermatrix for the classes image
		hclim.dimsize = nbDim;
		dimvar = 0;
		if (hclim.dims = (int *) calloc(hclim.dimsize, sizeof(int))) {
			hclim.dims[dimvar++] = height;
			hclim.dims[dimvar++] = width;
			hclim.dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		hclim.size = height * width * nbComp;
		hclim.R = (*classes_image);
		hclim.it = 0;
	} else {
		return(1);
	}

	// ........................................................................
	// Filling classes image
	// Call to a C++ function from fuzzy kmeans library

	fuzzyKmeansInit(nbComp, nbClasses, nbIm);
	err = fuzzyKmeansClassify(hproto, hmask, weights, fuzzyness, threshold1, threshold2, himages, hclim);
	fuzzyKmeansDeinit();


	// ........................................................................
	// Memory desallocation
	for (k=0 ; k<nbIm ; k++) {
		if (himages[k].dims) {
			free(himages[k].dims);
		}
	}
	if (hproto.dims) {
		free(hproto.dims);
	}
	if (hmask.dims) {
		free(hmask.dims);
	}
	if (hclim.dims) {
		free(hclim.dims);
	}

	return(err);
}


/*********************************************************************************************************************/
/*   Fuzzy k-means method classification phase function for a 3D image                                               */
/*********************************************************************************************************************/

int classify_fuzzykmeans_3D(double *proto, int nbIm, int nbClasses, HyperMat mask, double *weights, double fuzzyness,		double threshold1, double threshold2, HyperMat images[NBIMAGES_MAX], HyperMat *classes_image) {

	int i;
	int err = 0;
	const int nbDim = 3;
	const int nbComp = 1;
	int dimvar = 0;
	HyperMat hproto;


	// ........................................................................
	// Create an hypermatrix for the protoype
	hproto.dimsize = nbDim;
	dimvar = 0;
	if (hproto.dims = (int *) calloc(hproto.dimsize, sizeof(int))) {
		hproto.dims[dimvar++] = nbIm;
		hproto.dims[dimvar++] = nbClasses;
		hproto.dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	hproto.size = nbIm * nbClasses * nbComp;
	hproto.R = proto;
	hproto.it = 0;

	// ........................................................................
	// Memory allocation for the classes image
	classes_image->size = images[0].size;
	classes_image->dimsize = images[0].dimsize;
	if (classes_image->dims = (int *) calloc(classes_image->dimsize, sizeof(int))) {
		for (i=0 ; i<classes_image->dimsize ; i++) {
			classes_image->dims[i] = images[0].dims[i];
		}
	} else {
		return 1;
	}
	if (!(classes_image->R = (double *) calloc(classes_image->size, sizeof(double)))) {
		return 1;
	}
	classes_image->it = 0;


	// ........................................................................
	// Filling classes image
	// Call to a C++ function from fuzzy kmeans library

	fuzzyKmeansInit(nbComp, nbClasses, nbIm);
	err = fuzzyKmeansClassify(hproto, mask, weights, fuzzyness, threshold1, threshold2, images, *classes_image);
	fuzzyKmeansDeinit();


	// ........................................................................
	// Memory desallocation
	if (hproto.dims) {
		free(hproto.dims);
	}

	return(err);
}


/*********************************************************************************************************************/
/* Fuzzy k-means method classification phase function for a 4D image                                                 */
/*********************************************************************************************************************/

int classify_fuzzykmeans_4D(HyperMat proto, HyperMat mask, double *weights, double fuzzyness, double threshold1,
	double threshold2, HyperMat images[NBIMAGES_MAX], HyperMat *classes_image) {

	int i,k;
	int err = 0;

	int nbIm = proto.dims[0];
	int nbClasses = proto.dims[1];
	int nbComp = proto.dims[2];


	// ........................................................................
	// Memory allocation for the classes image
	classes_image->size = mask.size;
	classes_image->dimsize = mask.dimsize;
	if (classes_image->dims = (int *) calloc(classes_image->dimsize, sizeof(int))) {
		for (i=0 ; i<classes_image->dimsize ; i++) {
			classes_image->dims[i] = mask.dims[i];
		}
	} else {
		return 1;
	}
	if (!(classes_image->R = (double *) calloc(classes_image->size, sizeof(double)))) {
		return 1;
	}
	classes_image->it = 0;

	// ........................................................................
	// Filling classes image
	// Call to a C++ function from fuzzy kmeans library

	fuzzyKmeansInit(nbComp, nbClasses, nbIm);
	err = fuzzyKmeansClassify(proto, mask, weights, fuzzyness, threshold1, threshold2, images, *classes_image);
	fuzzyKmeansDeinit();


	return(err);
}




