
/******************************************************************************/
/* Fonctions used to carry out the Kohonen SOM method training phase          */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
//#include <kohonenlib.h>
#include "int_imkohonentrain.h"

/******************************************************************************/
/* Interface Scilab<->imkohonentrain                                          */
/******************************************************************************/
/* Creation : 07/05/2004 (V. POMMEREUL)                                       */
/* Last modification :                                                        */
/******************************************************************************/
int int_imkohonentrain(char *fname)
{
	static int l_dimension, m_dimension, n_dimension,
		l_learningData, m_learningData, n_learningData,
		l_nbClasses, m_nbClasses, n_nbClasses,
		l_nbIter, m_nbIter, n_nbIter,
		m_prototype, n_prototype;
	double *l_prototype;
	HyperMat h_learningData, h_prototype;

	int minlhs = 1, maxlhs = 1;
	int minrhs = 4, maxrhs = 4;

	int ierr=0;

	int ind_var=1;
	int ind_res=1;

	int dim, nbCl, nbIter;
	double *ldata;

	// Verify number of arguments
	CheckRhs(minrhs, maxrhs); // Input arguments
	CheckLhs(minlhs, maxlhs); // Output arguments

	// Get right hand side arguments
	GetRhsVar(ind_var++, "i", &m_dimension, &n_dimension, &l_dimension);
	dim = *istk(l_dimension);

	if (dim == 2) {
		// Get 2D learning data
		GetRhsVar(ind_var++, "d", &m_learningData, &n_learningData, &l_learningData);
		ldata = stk(l_learningData);
	} else {
		if (dim == 3){
			// Get 3D learning data
			GetRhsHMat(ind_var++, &h_learningData);
		} else {
			Scierror(999,"%s : Internal error \r\n",fname);
		}
	}

	GetRhsVar(ind_var++, "i", &m_nbClasses, &n_nbClasses, &l_nbClasses);
	GetRhsVar(ind_var++, "i", &m_nbIter, &n_nbIter, &l_nbIter);

	nbCl = *istk(l_nbClasses);
	nbIter = *istk(l_nbIter);

	if (dim == 2) {
		ierr = train_kohonen_mat(ldata, m_learningData, n_learningData, nbCl, nbIter, &l_prototype);
	} else {
		ierr = train_kohonen_hypmat(h_learningData, nbCl, nbIter, &h_prototype);
	}

	// In case of error
	if(ierr!=0) {
		Scierror(999,"%s : Internal error %i \r\n",fname, ierr);
	}

	// Return prototype
	if (dim == 2) {
		CreateVarFromPtr(ind_var, "d", &n_learningData, &nbCl, &l_prototype);
		if (l_prototype) {
			free(l_prototype);
		}
		LhsVar(ind_res++)=ind_var;
	} else {
		CreateHMat(ind_var, &h_prototype);
		LhsVar(ind_res++) = ind_var;
		// Memory desallocation
		if (h_prototype.dims) {
			free(h_prototype.dims);
		}
		if (h_prototype.R) {
			free(h_prototype.R);
		}
	}

	return(0);
}


/******************************************************************************/
/* Kohonen SOM method training phase function for a 2D or 3D image            */
/******************************************************************************/

int train_kohonen_mat(double *learningData, int nbPoints, int nbFunc, int nbClasses,
						int nbIter, double **prototype)
{
	const int nbDim = 3;
	const int nbComp = 1;
	int dimvar = 0;
	HyperMat proto;
	HyperMat ld;
	int err = 0;

	// ........................................................................
	// Memory allocation for the prototype
	if (*prototype = (double*) calloc(nbClasses * nbFunc * nbComp, sizeof(double))) {

		// Create an hypermatrix for the prototype
		proto.dimsize = nbDim;
		if (proto.dims = (int *) calloc(proto.dimsize, sizeof(int))) {
			proto.dims[dimvar++] = nbFunc;
			proto.dims[dimvar++] = nbClasses;
			proto.dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		proto.size = nbFunc * nbClasses * nbComp;
		proto.R = (*prototype);
		proto.it = 0;

	} else {
		return(1);
	}


	// ........................................................................
	// Create an hypermatrix for learning data
	ld.dimsize = nbDim;
	dimvar = 0;
	if (ld.dims = (int *) calloc(ld.dimsize, sizeof(int))) {
		ld.dims[dimvar++] = nbPoints;
		ld.dims[dimvar++] = nbFunc;
		ld.dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	ld.size = nbPoints * nbFunc * nbComp;
	ld.R = learningData;
	ld.it = 0;

	// ........................................................................
	// Filling proto
	// Call to a C++ function from kohonen library

	kohonenInit(nbComp, nbClasses, nbFunc);
	err = kohonenTrain(ld, proto, nbIter);
	kohonenDeinit();

	// ........................................................................
	// Free memory
	if (proto.dims) {
		free(proto.dims);
	}
	if (ld.dims) {
		free(ld.dims);
	}

	return(err);
}

/******************************************************************************/
/* Kohonen SOM method training phase function for a 4D image                  */
/******************************************************************************/

int train_kohonen_hypmat(HyperMat learningData, int nbClasses, int nbIter, HyperMat *prototype) {

	const int nbDim = 3;
	int dimvar = 0;
	int nbFunc, nbComp;
	int err = 0;

	nbFunc = learningData.dims[1];
	nbComp = learningData.dims[2];

	// ........................................................................
	// Filling prototype
	prototype->dimsize = nbDim;
	if (prototype->dims = (int *) calloc(prototype->dimsize, sizeof(int))) {
		prototype->dims[dimvar++] = nbFunc;
		prototype->dims[dimvar++] = nbClasses;
		prototype->dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	prototype->size = nbFunc * nbClasses * nbComp;
	prototype->it = 0;
	if (!(prototype->R = (double *) calloc(prototype->size, sizeof(double)))) {
		return 1;
	}

	// ........................................................................
	// Call to a C++ function from kohonen library

	kohonenInit(nbComp, nbClasses, nbFunc);
	err = kohonenTrain(learningData, *prototype, nbIter);
	kohonenDeinit();

	return err;

}


