/******************************************************************************/
/* Functions used to label all closed areas of an image from Scilab           */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_imlabel.h"


/******************************************************************************/
/* Interface Scilab <-> imlabel                                               */
/******************************************************************************/
/* Creation : 09/04/2002 (V. COUVERT)                                         */
/* Last modification : 09/06/2005 (M. TALLUR)                                 */
/******************************************************************************/
int int_imlabel(char *fname)
{
	int l_image, m_image, n_image,
	    m_label_image, n_label_image,
	    l_connectivity, m_connectivity, n_connectivity,
	    l_nb_objects, m_nb_objects=1, n_nb_objects=1,
	    l_vertical_parsing, m_vertical_parsing, n_vertical_parsing;
	double * l_label_image;

	int ierr=0;

	int minlhs=2, maxlhs=2, minrhs=2, maxrhs=3;
	
	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Ouput arguments 

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "i", &m_connectivity, &n_connectivity, &l_connectivity);
	GetRhsVar(3, "i", &m_vertical_parsing, &n_vertical_parsing, &l_vertical_parsing);

	CreateVar(4, "i", &m_nb_objects, &n_nb_objects, &l_nb_objects);

	label(stk(l_image), m_image, n_image, *istk(l_connectivity), &l_label_image, istk(l_nb_objects), &ierr, *istk(l_vertical_parsing));
	
	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(5,"d", &m_image, &n_image, &l_label_image);

	if (l_label_image) {
		free(l_label_image);
	}
	

	LhsVar(1)=5;
	LhsVar(2)=4;

	return(0);
}

/******************************************************************************/
/* Labelling main function                                                    */
/******************************************************************************/
/* Creation : 09/04/2002 (V. COUVERT)                                         */
/* Last modification : 09/06/2005 (M. TALLUR)                                 */
/******************************************************************************/
int label(double *image, int width, int height, int connectivity, 
		  double **label_image, int *nb_objects, int *ierr, int vertical_parsing)
{
	int x=0, y=0, *coo1, *coo2, dim1, dim2, u;
	long i=0;
	double label=0, labelCurrentPixel;
	double val;

	struct imlabel_synonym *head = NULL, *tail = NULL, *current = NULL;
	struct imlabel_list **syns_lists_list;
	long *syns_table; // syns_table[synonym] = value
	long *label_offset; // label_offset[i] = u : u values were skipped before i
	long offset=0;

	/* Memory allocation */
	if (!(*label_image = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}

	for (y=0; y<height; y++)
		for (x=0; x<width; x++)
			(*label_image)[x+y*width]=0;


	/* 
	   The algorithm consists in parsing the picture from the upper left corner 
	   to the bottom right corner. Depending on the required connectivity, each pixel
	   is compared to 2 or 4 or its neighbours, provided they haven't been parsed yet
	   - that is to say, each pixel is parsed only once. 

       For instance, if we find out the current pixel is the same as the pixel on the right, we consider
	   they are part of the same object. When a hole occurs, we raise the objects counter.
	   Later, it's possible two separate objects turn out to be only one, when the two parts
	   merge again a couple of lines later.

       When this happens, we thus make use of a table of synonyms (initialized to the identity).
	   When the algorythm is over, we just have to replace all synonyms with the same value
	*/

	if ((connectivity != 4) && (connectivity !=8))
		// illegal parameter
		return (-1);

	if (vertical_parsing)
	{
		coo1 = &y; coo2 = &x;
		dim1 = height; dim2 = width;
	}
	else
	{
		coo1 = &x; coo2 = &y;
		dim1 = width ; dim2 = height;
	}

	for ((*coo2)=0; (*coo2)<dim2; (*coo2)++)
		for ((*coo1)=0; (*coo1)<dim1; (*coo1)++)
		{
			if ((val = image[x+y*width]))
			// not a black dot
			{
				if (!(*label_image)[x+y*width]){ 
					(*label_image)[x+y*width] = ++label;
					imlabel_addSynonym(&head, &tail, (long)label, (long)label);
				}
				labelCurrentPixel = (*label_image)[x+y*width];
				
				for (u=0; u<(connectivity == 4 ? 2 : 4); u++)
				{
					if ((x<width-neighboffset_x[u]) && (x>=-neighboffset_x[u]) && (y<height-neighboffset_y[u]) && (val == image[x+neighboffset_x[u]+(y+neighboffset_y[u])*width]))
						if ((*label_image)[x+neighboffset_x[u]+(y+neighboffset_y[u])*width])
						{
							if ((*label_image)[x+neighboffset_x[u]+(y+neighboffset_y[u])*width] != labelCurrentPixel)
								imlabel_addSynonym(&head, &tail, labelCurrentPixel, (*label_image)[x+neighboffset_x[u]+(y+neighboffset_y[u])*width]); 
								// two objects merge, thus they're the same : labels are synonyms
						}
						else // neigbour has no label yet
							(*label_image)[x+neighboffset_x[u]+(y+neighboffset_y[u])*width] = labelCurrentPixel;
				}	
			}
		}


	// for each label, let's compute a list containing the synonyms defined above
	imlabel_convertSynsListToSynsListsList(&syns_lists_list, head, label);

	// for each label, let's compute all the synonyms taking recursive definitions into account
	imlabel_spreadSynonyms(syns_lists_list, label);

	// now for a quicker replacement, let's put the best synonym (lowest value) of each lable in a table
	imlabel_convertSynsListsListToSynsTable(&syns_table, syns_lists_list, label);


	// labels numbering may now contain 'holes'
	if (!(label_offset = (long *) malloc(sizeof(long)*((long)label+1))))
	{
		*ierr=1;
		return(-1);
	}

	// label_offset[x] contains the number of labels that were skipped / replaced
	// before the x-ieth label and that will have to be deduced from each 
	// label value to prevent holes from occuring in the numbering
 
	label_offset[0]=0; label_offset[1]=0;

	for (i=1; i<label; i++)
	  {

	    if (syns_table[i] != i) // label has to be replaced ? -> hole in the numbering
		 offset++;
	    label_offset[i+1] = offset;
	    syns_table[i] -= label_offset[syns_table[i]]; // we can make the deduction right now as label_table[i] <= i
	  }

	if ((int)label) 
	{
		if (syns_table[(long)label] != label) offset++; 
	  	syns_table[(long)label] -= label_offset[syns_table[(long)label]];
	}

	for (y=0; y<height; y++)
		for (x=0; x<width; x++)
			if (syns_table[(long)(*label_image)[x+y*width]])
				(*label_image)[x+y*width] = syns_table[(long)(*label_image)[x+y*width]];

	// Freeing all the formerly allocated memory... Yes, this function is both time & memory consuming... It should work though ! :)
	imlabel_emptyListsList(syns_lists_list, label);
	imlabel_emptyList(head);
	free(label_offset);
	free(syns_table);

	// Number of objects in the picture : number of zones  minus  number of equivalent zones
	*nb_objects = label-offset;
	//printf("nb objets : %d \n",*nb_objects);
	*ierr = 0;
	return 0;
}

/*************************************/
/* Functions to manage lists         */
/*************************************/
/* Creation : 09/06/2005 (M. TALLUR) */
/*************************************/

// Beware : most likely not optimal at all

// Warning : the 'next' field in the following structs must always be the first
// as only a single function is used to free two different kinds of chained lists

static bool imlabel_belongsToList(long value, struct imlabel_integer *head)
{
	struct imlabel_integer *current = head;
	bool found = false;
	
	while (current && !found)
	{
		if (current->value == value)
			found = true;
		current = current->next;
	}

	return found;
}

// using the following doesn't seem to make much difference speed-wise contrary to the latter
static bool imlabel_coupleBelongsToList(long value, long syn, struct imlabel_synonym *head)
{
	struct imlabel_synonym *current = head;
	bool found = false;

	while (current && !found)
	{
		if (current->value == value)
			if (current->synonym == syn)
				found = true;
		current = current->next;
	}

	return found;
}

static void imlabel_addSynonym(struct imlabel_synonym **head, struct imlabel_synonym **tail, long value1, long value2)
{
	struct imlabel_synonym *new_tail=NULL;
	long value, synonym;

	if (value1<value2)
	{
		value = value1;
		synonym = value2;
	}
	else
	{
		value = value2;
		synonym = value1;
	}	

	if ((!imlabel_coupleBelongsToList(value, synonym, *head)) && (new_tail = (struct imlabel_synonym *) malloc(sizeof(struct imlabel_synonym))))
	{
		new_tail->value = value;
		new_tail->synonym = synonym;
		new_tail->next = NULL;

		if (*tail)
		  (*tail)->next = new_tail;

		*tail = new_tail;
	}

	if (!*head)
		*head = new_tail;
}

static bool imlabel_addInteger(struct imlabel_integer **head, struct imlabel_integer **tail, long value, int check)
// returns "true" if integer added
{
	struct imlabel_integer *new_tail;

	if ((!check) || (!imlabel_belongsToList(value, *head)))
	{

		if ((new_tail = (struct imlabel_integer *) malloc(sizeof(struct imlabel_integer))))
		{
			new_tail->value = value;
			new_tail->next = NULL;

			if (*tail)
			  (*tail)->next = new_tail;

			*tail = new_tail;
		}

		if (!*head)
			*head = new_tail;

		return true;
	}
	else
		return false;
}


static bool imlabel_convertSynsListToSynsListsList(struct imlabel_list ***newlist, struct imlabel_synonym *head, long count)
// returns false is a memory allocation failed
{
	struct imlabel_synonym *current;
	struct imlabel_list *nl;

	if ((*newlist = (struct imlabel_list **)malloc(sizeof(struct imlabel_list *)*(count+1))))
	{
		long i;

		for (i=0; i<=count; i++)
		{ 
			if (((*newlist)[i] = (struct imlabel_list *)malloc(sizeof(struct imlabel_list))))
			{
				(*newlist)[i]->head = NULL; (*newlist)[i]->tail = NULL;
			}
			else
				return false;
		}

		current = head;
		while(current)
		{ 
			nl = (*newlist)[current->synonym];
			imlabel_addInteger(&(nl->head), &(nl->tail), current->value, 1);
			current = current->next;
		}

		return true;
	}
	else
		return false;
}


static long imlabel_minimum(struct imlabel_integer *head)
{
	struct imlabel_integer *current = head;
	long minimum = 0;

	while(current)
	{
		if (minimum)
		{
			if (minimum > current->value)
				minimum = current->value;
		}
		else
			minimum = current->value;
		current = current->next;
	}

	return minimum;
}

static void imlabel_spreadSynonyms(struct imlabel_list **list, long count)
{
	// it turned out what we're actually doing is computing the "fermeture transitive du graphe"
	long i, u, nbboucles;
	struct imlabel_integer *current, *current2;
	bool list_modified = true;
	struct imlabel_list *list_current_value; // to speed things up, but maybe gcc already does the optimization itself ?
	int *arcs; // <--- on pourrait caser 32 bool l dedans,  faire pour optimiser utilisation memoire
	
	if ((arcs = malloc(sizeof(int)*(count+1)*(count+1))))
	{
		for (i=0; i<=count; i++)
			for (u=0; u<=count; u++)
				arcs[i + u*count] = 0;
			
		for (i=1; i<=count; i++)
		{
			current = list[i]->head;
			while (current)
			{
				arcs[i + count*current->value] = 1;
				current = current->next;
			}
		}
	}
	else
		Scierror(999,"imlabel : not enough memory ! Bug the person who wrote this to optimize it !\r\n");
	nbboucles = 0;
	
	while (list_modified && arcs)
	{
	
		i=count; list_modified = false;
		while (i>=1)
		{
			current = list[i]->head;
			while (current)
			{

				list_current_value = list[current->value];	
				if (!arcs[current->value + i*count])
				{
					arcs[current->value + i*count] = 1;
					if (imlabel_addInteger(&(list_current_value->head), &(list_current_value->tail), i, 0))
						list_modified = true;
				}
				
				current2 = current->next; //list[i]->head;
				while (current2)
				{
				nbboucles++;
					if (!arcs[current->value + count*current2->value])
					{
						arcs[current->value + count*current2->value] = 1;
						if (imlabel_addInteger(&(list_current_value->head), &(list_current_value->tail), current2->value, 0))					
							list_modified = true;
					}
					current2 = current2->next;
				}
				
				/*current2 = list_current_value->head;
				while (current2)
				{
				nbboucles++;
					if (!arcs[i + current2->value*count])
					{
						arcs[i + current2->value*count] = 1;
						if (imlabel_addInteger(&(list[i]->head), &(list[i]->tail), current2->value, 0))
							list_modified = true;
					}
					current2 = current2->next;
				}*/

				current = current->next;
			}
			i--;
		}
	}
	
	if (arcs) free(arcs);
	
}

static void imlabel_convertSynsListsListToSynsTable(long **tab, struct imlabel_list **list, long count)
{
	long i;
	if (((*tab) = (long *)malloc(sizeof(long)*(count+1))))
	{
		(*tab)[0] = 0;
		for (i=1; i<=count; i++)
			(*tab)[i] = imlabel_minimum(list[i]->head);
	}
}

static void imlabel_emptyList(struct imlabel_synonym *head)
{
	struct imlabel_synonym *current, *next;
	current = head;
  
	while (current)
    	{
		next = current->next;
		free(current); 
	 	current = next;
	}
}

static void imlabel_emptyListsList(struct imlabel_list **list, long count)
{
	long i;

	for (i=0; i<=count; i++)
	{
		imlabel_emptyList(list[i]->head);
		free(list[i]);
	}
	free(list);
}
