/******************************************************************************/
/* Functions used to analyse shapes in an image from Scilab                   */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include <math.h>

#include "int_imshapeanalysis.h"

#define MAX_PARAMETERS 5
#define PI 3.1415927

/******************************************************************************/
/* Interface Scilab <-> imshapeanalysis                                       */
/******************************************************************************/
/* Creation : 16/05/2002 (V. COUVERT)                                         */
/* Last modification : 04/06/2002 (V. COUVERT)                                */
/******************************************************************************/
int int_imshapeanalysis(char *fname)
{
	int l_image, m_image, n_image,
	    l_parameters, m_parameters, n_parameters,
	    l_parameters_nb, m_parameters_nb, n_parameters_nb,
	    l_objects_nb, m_objects_nb, n_objects_nb,
	    m_results, n_results;
	double *l_results;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=4, maxrhs=4;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);

	GetRhsVar(2, "i", &m_parameters, &n_parameters, &l_parameters);

	GetRhsVar(3, "i", &m_parameters_nb, &n_parameters_nb, &l_parameters_nb);

	GetRhsVar(4, "i", &m_objects_nb, &n_objects_nb, &l_objects_nb);

	analyse_shape(stk(l_image), m_image, n_image, istk(l_parameters), *istk(l_parameters_nb), *istk(l_objects_nb), &l_results, &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}
	//Size of output variable
	m_results=*istk(l_objects_nb);
	n_results=*istk(l_parameters_nb);

	CreateVarFromPtr(5,"d", &m_results, &n_results, &l_results);

	if (l_results) {
		free(l_results);
	}

	LhsVar(1)=5;

	return(0);
}

/******************************************************************************/
/* Shape analysis function                                                    */
/******************************************************************************/
/* Creation : 16/05/2001 (V. COUVERT)                                         */
/* Last modification : 05/06/2002 (V. COUVERT)                                */
/******************************************************************************/
int analyse_shape(double *image, int width, int height, int *parameters, 
									int parameters_nb, int objects_nb, double **results, int *ierr)
{
	int i=0, x=0, y=0;
	int *nb_t_0,
	    *nb_t_45,
	    *nb_t_90,
	    *nb_t_135,
	    *xsum,
	    *ysum,
	    *area;
	double 	*Si,		//second order shape moments
		*Sj,
		*Sij;		//second order shape moments
	double *Gx,		//centers of mass
	       *Gy;
	double *val_p1;
	double *val_p2;
	double delta;			//for the proper values


	// Memory allocation
	//------------------
	if (!((nb_t_0)		= (int*)calloc(1, sizeof(int)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if (!((nb_t_45)		= (int*)calloc(1, sizeof(int)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if (!((nb_t_90)		= (int*)calloc(1, sizeof(int)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if (!((nb_t_135)	= (int*)calloc(1, sizeof(int)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if (!((xsum)		= (int*)calloc(1, sizeof(int)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if (!((ysum)		= (int*)calloc(1, sizeof(int)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if (!((area)		= (int*)calloc(1, sizeof(int)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if(!((Si)		= (double*)calloc(1,sizeof(double)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if(!((Sj)		= (double*)calloc(1,sizeof(double)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if(!((Sij)		= (double*)calloc(1,sizeof(double)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if(!((val_p1)		= (double*)calloc(1,sizeof(double)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if(!((val_p2)		= (double*)calloc(1,sizeof(double)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if(!((Gx)		= (double*)calloc(1,sizeof(double)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}
	if(!((Gy)		= (double*)calloc(1,sizeof(double)*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}



	// Memory allocation
	//------------------
	if (!((*results)	= (double*)calloc(1,sizeof(double)*parameters_nb*objects_nb)))
	{
		*ierr=1;
		return(*ierr);
	}

	// Main loop
	//----------
	for(y=0;y<height;y++) // Loop on columns
	{
		for(x=0;x<width;x++) // Loop on lines
		{
			// Level zero is supposed to be background
			// and is not considered as an object
			if((*(image+x+y*width))!=0)	//when object met
			{
				// Surface
				area[(int)(*(image+x+y*width))-1]++;
				// xcenter
				xsum[(int)(*(image+x+y*width))-1]=xsum[(int)(*(image+x+y*width))-1]+x+1;
				// ycenter
				ysum[(int)(*(image+x+y*width))-1]=ysum[(int)(*(image+x+y*width))-1]+y+1;

				// Perimeter
				if(*(parameters+2)==1)	//if required
				{
					if(x<width-1 && y<height-1 && x>0 && y>0)
					{
						if(*(image+x-1+y*width) != *(image+x+y*width))
							nb_t_0[(int)(*(image+x+y*width))-1]++;
						if(*(image+x+(y+1)*width) != *(image+x+y*width))
							nb_t_90[(int)(*(image+x+y*width))-1]++;
						if(*(image+x-1+(y+1)*width) != *(image+x+y*width))
							nb_t_45[(int)(*(image+x+y*width))-1]++;
						if(*(image+x+1+(y+1)*width) != *(image+x+y*width))
							nb_t_135[(int)(*(image+x+y*width))-1]++;
					}
				}

			}
		} // End of lines loop
	} // End of columns loop

	//mass center of the objects, without any requirement
	//---------------------------------------------------
	for(i=1;i<=objects_nb;i++)
	{
		Gx[i-1]=(double)xsum[i-1]/((double)area[i-1]);
		Gy[i-1]=(double)ysum[i-1]/((double)area[i-1]);
	}


//------------------------
//response to requirements
//------------------------
	// Surface
	//--------
	if((*parameters)==1)
	{
		for(i=1;i<=objects_nb;i++)
		{
	(*results)[(*(parameters+1)-1)*objects_nb+i-1]=area[i-1];
		}
	}
	// Perimeter
	//----------
	if(*(parameters+2)==1)
	{
		for(i=1;i<=objects_nb;i++)
		{
			(*results)[(*(parameters+3)-1)*objects_nb+i-1]=PI/4*(nb_t_0[i-1]+nb_t_90[i-1]+(nb_t_45[i-1]+nb_t_135[i-1])/sqrt(2));
		}
	}
	// Compacity
	//----------
	if(*(parameters+4)==1)
	{
		for(i=1;i<=objects_nb;i++)
		{
			(*results)[(*(parameters+5)-1)*objects_nb+i-1]=4*PI*((*results)[(*(parameters+1)-1)*objects_nb+i-1])/( ((*results)[(*(parameters+3)-1)*objects_nb+i-1]) * ((*results)[(*(parameters+3)-1)*objects_nb+i-1]) );
		}
	}
	// xcenter
	//--------
	if(*(parameters+6)==1)
	{
		for(i=1;i<=objects_nb;i++)
		{
			(*results)[(*(parameters+7)-1)*objects_nb+i-1]=Gx[i-1];
		}
	}
	// ycenter
	//--------
	if(*(parameters+8)==1)
	{
		for(i=1;i<=objects_nb;i++)
		{
			(*results)[(*(parameters+9)-1)*objects_nb+i-1]=Gy[i-1];
		}
	}
	// Principal axises
	//-----------------
	if(*(parameters+10)==1 || *(parameters+12)==1 || *(parameters+14)==1)
	{
		//loops for the calculation of the second order moments matrix terms
		for (y=0;y<height;y++)
		{
			for (x=0;x<width;x++)
			{
				if((*(image+x+y*width))!=0)
				{
Si[(int)(*(image+x+y*width)-1)]=Si[(int)(*(image+x+y*width)-1)]+(x-(Gx[(int)(*(image+x+y*width))-1]))*(x-(Gx[(int)(*(image+x+y*width))-1]));
Sj[(int)(*(image+x+y*width))-1]=Sj[(int)(*(image+x+y*width))-1]+(y-(Gy[(int)(*(image+x+y*width))-1]))*(y-(Gy[(int)(*(image+x+y*width))-1]));
Sij[(int)(*(image+x+y*width))-1]=Sij[(int)(*(image+x+y*width))-1]+(x-(Gx[(int)(*(image+x+y*width))-1]))*(y-(Gy[(int)(*(image+x+y*width))-1]));
				} //end of if loop
			} //end of x loop
		} //end of y loop

		//final values of the second order moments
		//the moment of inertia matrix is such as :
		//	Si  Sij
		//	Sij Sj
		for(i=1;i<=objects_nb;i++)	//loop on number of objects
		{
			Si[i-1]  = Si[i-1]/area[i-1];
			Sj[i-1]  = Sj[i-1]/area[i-1];
			Sij[i-1] = Sij[i-1]/area[i-1];
		}

		//calculation of the proper values: axises of the objects
		for(i=0;i<objects_nb;i++)
		{
			delta     = (Si[i])*(Si[i])+(Sj[i])*(Sj[i])+4*(Sij[i])*(Sij[i])-2*(Si[i])*(Sj[i]);
			val_p1[i] = (Si[i]+Sj[i]+sqrt(delta))/2;	//such defined: lambda1>lambda2
			val_p2[i] = (Si[i]+Sj[i]-sqrt(delta))/2;
		}

		//entering values in **results if required
		if(*(parameters+10)==1)		//if lambda 1 is required
		{
			for(i=0;i<objects_nb;i++)
			{
				(*results)[(*(parameters+11)-1)*objects_nb+i]=val_p1[i];
			}
		}
		if(*(parameters+12)==1)		//if lambda 1 is required
		{
			for(i=0;i<objects_nb;i++)
			{
				(*results)[(*(parameters+13)-1)*objects_nb+i]=val_p2[i];
			}
		}

		//angle theta (in degrees)
		if(*(parameters+14)==1)		//if angle theta required
		{
			for(i=0;i<objects_nb;i++)
			{
			(*results)[(*(parameters+15)-1)*objects_nb+i]=180*atan((Si[i]-val_p1[i])/Sij[i])/PI;
			}
		}
	}//end of if loop

	//Memory freeing
	free(nb_t_0);
	free(nb_t_45);
	free(nb_t_90);
	free(nb_t_135);
	free(xsum);
	free(ysum);
	free(area);
	free(Si);
	free(Sj);
	free(Sij);
	free(val_p1);
	free(val_p2);
	free(Gx);
	free(Gy);

	*ierr=0;
	return(0);
}

/******************************************************************************/
/* Contents of *parameters variable                                           */
/******************************************************************************/
/* *(parameters+0) : compute area or not                                      */
/* *(parameters+1) : position of area in call                                 */
/* *(parameters+2) : compute perimeter or not                                 */
/* *(parameters+3) : position of perimeter in call                            */
/* *(parameters+4) : compute compacity or not                                 */
/* *(parameters+5) : position of compacity in call                            */
/* *(parameters+6) : compute xcenter or not                                   */
/* *(parameters+7) : position of xcenter in call                              */
/* *(parameters+8) : compute ycenter or not                                   */
/* *(parameters+9) : position of ycenter in call                              */
/* *(parameters+10) : compute lambda1 or not                                  */
/* *(parameters+11) : position of lambda1 in call                             */
/* *(parameters+12) : compute lambda2 or not                                  */
/* *(parameters+13) : position of lambda2 in call                             */
/* *(parameters+14) : compute theta or not                                    */
/* *(parameters+15) : position of theta in call                               */
/******************************************************************************/

/******************************************************************************/
/* Contents of **results variable                                             */
/******************************************************************************/
/*            parameter n1 parameter n2 parameter n3 ...                   */
/* object n1     value         value         value                           */
/* object n2     value         value         value                           */
/* ...             ...           ...           ...                            */
/******************************************************************************/

