/******************************************************************************/
/* Functions used to get edges of an image from Scilab                        */
/* Using SHEN & CASTAN method                                                 */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "stack-c.h"

#include "int_imshencastan.h"

/******************************************************************************/
/* Interface Scilab <-> imshencastan                                          */
/******************************************************************************/
/* Creation : 23/04/2002 (V. COUVERT)                                         */
/* Last modification : 30/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imshencastan(char *fname)
{
	int l_image, m_image, n_image,
	    l_alpha, m_alpha, n_alpha,
	    l_direction, m_direction, n_direction,
	    l_type, m_type, n_type,
		l_mask, m_mask, n_mask,
	    m_filtered_image, n_filtered_image;
	double *l_filtered_image;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=5, maxrhs=5;

	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Ouput arguments 

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_alpha, &n_alpha, &l_alpha);
	GetRhsVar(3, "i", &m_direction, &n_direction, &l_direction);
	GetRhsVar(4, "c", &m_type, &n_type, &l_type);
	GetRhsVar(5, "d", &m_mask, &n_mask, &l_mask);

	shencastan(stk(l_image), m_image, n_image, *stk(l_alpha), *istk(l_direction),
		*cstk(l_type), stk(l_mask), &l_filtered_image,&ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(5,"d", &m_image, &n_image, &l_filtered_image);

	if (l_filtered_image) {
		free(l_filtered_image);
	}

	LhsVar(1)=5;

	return(0);
}

/******************************************************************************/
/* filtered image computing                                                   */
/******************************************************************************/
/* Creation : 23/04/2002 (V. COUVERT)                                         */
/* Last modification : 30/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int shencastan(double *image, int width, int height, double alpha, int direction,
			   char type, double *mask, double **filtered_image, int *ierr)
{
	int x, y;
	double *splus, *sminus;

	// Memory allocation 
	if (!(*filtered_image = (double*)calloc(width*height, sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}

	if(direction==0)
	{
		// Memory allocation 
		if (!(splus = (double*)calloc(width, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}
		if (!(sminus = (double*)calloc(width, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}

		for(y=0 ; y<height ; y++)
		{
			// Initialisation 
			*splus = image[y*width];
			sminus[width-1] = image[width-1+y*width];

			for(x=1 ; x<width ; x++) {

				splus[x] = splus[x-1] + alpha * (image[x+y*width] - splus[x-1]);
			}

			for(x=width-1 ; x>=0 ; x--){

				if(x < width-1) {
					//Because first value has been initialised
					sminus[x] = sminus[x+1] + alpha * (image[x+y*width] - sminus[x+1]);
				}

				if(type=='g') {	

					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {
					
						(*filtered_image)[x+y*width] = splus[x] - sminus[x];
					
					} else {

						(*filtered_image)[x+y*width] = image[x+y*width];
					}
				
				} else {

					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {
					
						(*filtered_image)[x+y*width] = 0.5 * (splus[x] + sminus[x]);
					
					} else {

						(*filtered_image)[x+y*width] = image[x+y*width];
					}
				}
			}
		}
	
	} else if(direction==45) {

		// Memory allocation 
		if (!(splus = (double*)calloc(height*width, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}
		if (!(sminus = (double*)calloc(height*width, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}

		/* s+ and s- initialisation */
		for(x=0 ; x<width ; x++)
		{
			splus[x+(height-1)*width] = image[x+(height-1)*width];
			sminus[x] = image[x];
		}
		for(y=0 ; y<height ; y++)
		{
			splus[y*width] = image[y*width];
			sminus[(y+1)*width-1] = image[(y+1)*width-1];
		}

		/* Computation loop for s+ */
		for(x=1 ; x<width ; x++)
		{
			for(y=height-2 ; y>=0 ; y--)
			{
				splus[x+y*width] = splus[x-1+(y+1)*width] 
					+ alpha * (image[x+y*width] - splus[x-1+(y+1)*width]);
			}
		}

		/* Computation loop for s- */
		for(x=width-2 ; x>=0 ; x--)
		{
			for(y=1 ; y<height ; y++)
			{
				sminus[x+y*width] = sminus[x+1+(y-1)*width]
					+ alpha * (image[x+y*width] - sminus[x+1+(y-1)*width]);
			}
		}

		/* filtered image computation */
		for(y=0 ; y<height ; y++)
		{
			for(x=0 ; x<width ; x++)
			{
				if(type == 'g')
				{
					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {

						(*filtered_image)[x+y*width] = splus[x+y*width] 
							- sminus[x+y*width];

					} else {

						(*filtered_image)[x+y*width] = image[x+y*width];
					}
				}
				else
				{
					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {

						(*filtered_image)[x+y*width] = 0.5 * ( splus[x+y*width] 
							+ sminus[x+y*width] );

					} else {

						(*filtered_image)[x+y*width] = image[x+y*width];
					}

				}
			}
		}

	} else if(direction==90) {
	
		/* Memory allocation */
		if (!(splus = (double*)calloc(height, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}
		if (!(sminus = (double*)calloc(height, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}

		for(x=0 ; x<width ; x++)
		{
			/* Initialisation */
			*splus = image[x];
			sminus[height-1] = image[x+(height-1)*width];

			for(y=1 ; y<height ; y++)
			{
				splus[y] = splus[y-1] + alpha * (image[x+y*width] - splus[y-1]);
			}
			for(y=height-1 ; y>=0 ; y--)
			{
				if(y<height-1)
				{
					sminus[y] = sminus[y+1] + 
						alpha * (image[x+y*width] - sminus[y+1]);
				}

				if(type=='g')
				{
					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {

						(*filtered_image)[x+y*width] = splus[y] - sminus[y];

					} else {
					
						(*filtered_image)[x+y*width] = image[x+y*width];
					}
				}
				else
				{
					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {

						(*filtered_image)[x+y*width] = 0.5 * (splus[y] + sminus[y]);
					} else {
					
						(*filtered_image)[x+y*width] = image[x+y*width];
					}
				}
			}
		}

	} else if(direction==135) {
	
		/* Memory allocation */
		if (!(splus = (double*)calloc(height*width, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}
		if (!(sminus = (double*)calloc(height*width, sizeof(double))))
		{
			*ierr=1;
			return(-1);
		}

		/* s+ and s- initialisation */
		for(x=0 ; x<width ; x++)
		{
			sminus[x+(height-1)*width] = image[x+(height-1)*width];
			splus[x] = image[x];
		}
		for(y=0 ; y<height ; y++)
		{
			splus[y*width] = image[y*width];
			sminus[(y+1)*width-1] = image[(y+1)*width-1];
		}

		/* Computation loop for s+ */
		for(x=1 ; x<width ; x++)
		{
			for(y=1 ; y<height ; y++)
			{
				splus[x+y*width] = splus[x-1+(y-1)*width]
					+ alpha * (image[x+y*width] - splus[x-1+(y-1)*width]);
			}
		}
		/* Computation loop for s- */
		for(x=width-2 ; x>=0 ; x--)
		{
			for(y=height-2 ; y>=0 ; y--)
			{
				sminus[x+y*width] = sminus[x+1+(y+1)*width]
					+ alpha * (image[x+y*width] - sminus[x+1+(y+1)*width] );
			}
		}
		/* filtered image computation */
		for(y=0 ; y<height ; y++)
		{
			for(x=0 ; x<width ; x++)
			{
				if(type=='g')
				{
					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {
	
						(*filtered_image)[x+y*width] = splus[x+y*width] - sminus[x+y*width];

					} else {
					
						(*filtered_image)[x+y*width] = image[x+y*width];
					}
				}
				else
				{
					// If the pixel is not masked 
					if(mask[x+y*width]!=0) {

						(*filtered_image)[x+y*width] = 0.5 * (splus[x+y*width] + sminus[x+y*width]);

					} else {
					
						(*filtered_image)[x+y*width] = image[x+y*width];
					}
				}
			}
		}
	}

	// Memory freeing 
	if (splus) {
		free(splus);
	}
	if (sminus) {
		free(sminus);
	}

	*ierr=0;
	return(0);
}


