/******************************************************************************/
/* Functions used to calculate a standard deviation image from Scilab         */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_imstandarddeviation.h"

/******************************************************************************/
/* Interface Scilab <-> imstandarddeviation                                   */
/******************************************************************************/
/* Creation : 16/03/2004 (V. POMMEREUL)                                       */
/* Last modification : 23/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imstandarddeviation(char *fname)
{
	// Input and output arguments
	int l_image, m_image, n_image,
	    l_kernel, m_kernel, n_kernel,
	    m_sdeviation_image, n_sdeviation_image,
	    l_mask, m_mask, n_mask;
	double *l_sdeviation_image;

	// Number of input and output arguments
	int minlhs=1, maxlhs=1;
	int minrhs=3, maxrhs=3;

	int ierr=0;

	int ind_var=1;
	int ind_res=1;

	double *image, *kernel, *mask;

	// Verify number of arguments
	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	GetRhsVar(ind_var++, "d", &m_image, &n_image, &l_image);
	GetRhsVar(ind_var++, "d", &m_kernel, &n_kernel, &l_kernel);
	GetRhsVar(ind_var++, "d", &m_mask, &n_mask, &l_mask);

	image = stk(l_image);
	kernel = stk(l_kernel);
	mask = stk(l_image);

	ierr = sdeviation_image(image, m_image, n_image, kernel, m_kernel, n_kernel, mask, &l_sdeviation_image);

	/* In case of error */
	if(ierr!=0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(ind_var,"d", &m_image, &n_image, &l_sdeviation_image);

	if (l_sdeviation_image) {
		free(l_sdeviation_image);
	}

	LhsVar(ind_res++)=ind_var;

	return(0);
}

/******************************************************************************/
/* Image standard deviation computation function                              */
/******************************************************************************/
/* Creation : 16/03/2004 (V. POMMEREUL)                                       */
/* Last modification :                                                        */
/******************************************************************************/
int sdeviation_image(double *image, int width, int height, double *kernel, int kernel_width, int kernel_height,
					 	double *mask, double **sdeviation_image) {

	double meanvalue, sdeviation;
	static int x, y, xscrut, yscrut, nb_val=0;
	double sum=0.0, sumsd = 0.0, temp = 0.0;
	int ierr = 0;
	int i;

	// Memory allocation
	if (!(*sdeviation_image = (double*)calloc(width*height, sizeof(double)))) {
		return(1);
	}


	// Filling standarddeviation image
	for(y=0 ; y<height ; y++)
	{
		for(x=0 ; x<width ; x++)
		{
			// If the pixel is not masked
			if(mask[x+y*width]!=0)
			{
				// Find the minimum
				for(yscrut=-kernel_height/2 ; yscrut<=kernel_height/2 ; yscrut++)
				{
					for(xscrut=-kernel_width/2 ; xscrut<=kernel_width/2 ; xscrut++)
					{
						// If value has to be count : kernel corresponding value is != 0
						if(kernel[xscrut+kernel_width/2+(yscrut+kernel_height/2)*kernel_width]!=0)
						{
							// If we point a pixel in the image (to take border in account) and the pixel is not masked
							if( (x+xscrut>=0) && (x+xscrut<width) && (y+yscrut>=0) && (y+yscrut<height) &&
									(mask[x+xscrut+(y+yscrut)*width]!=0) )
							{
								sum=sum+(image[x+y*width+xscrut+yscrut*width]);
								nb_val++;
							}
						}
					}
				}
				if(nb_val!=0) {
					meanvalue = sum/nb_val;
				} else {
					meanvalue = 0.0;
				}

				// Standard deviation computation
				for(yscrut=-kernel_height/2 ; yscrut<=kernel_height/2 ; yscrut++) {
					for(xscrut=-kernel_width/2 ; xscrut<=kernel_width/2 ; xscrut++) {

						// If value has to be count : kernel corresponding value is != 0
						if(kernel[xscrut+kernel_width/2+(yscrut+kernel_height/2)*kernel_width]!=0) {

							// If we point a pixel in the image (to take border in account) and the pixel is not masked
							if( (x+xscrut>=0) && (x+xscrut<width) && (y+yscrut>=0) && (y+yscrut<height) &&
									(mask[x+xscrut+(y+yscrut)*width]!=0) ) {

								temp = image[x + y*width + xscrut + yscrut*width] - meanvalue;
								temp = temp * temp;
								sumsd += temp;
							}
						}
					}
				}

				if(nb_val!=0) {
					sumsd = sqrt(sumsd/nb_val);
				} else {
					sumsd = 0.0;
				}

				(*sdeviation_image)[x+y*width] = sumsd;

				sum=0.0;
				sumsd = 0.0;
				nb_val=0;

			}
			else
			{
				(*sdeviation_image)[x+y*width]=*(image+x+y*width);
			}
		}
	}

	return ierr;
}
