/******************************************************************************/
/* Functions used to convert a xyz image to a L*a*b* image from Scilab        */
/******************************************************************************/
#include <math.h>
#include "stack-c.h"
#include "int_imxyz2lab.h"

/******************************************************************************/
/* Interface Scilab <-> imxyz2lab                                             */
/******************************************************************************/
/* Creation : 30/04/2002 (S. BONDU)                                           */
/* Last modification : 08/10/2002 (P. LOISEL)                                 */
/******************************************************************************/
int int_imxyz2lab(char *fname)
{
	int l_imageX, m_imageX, n_imageX,
	    l_imageY, m_imageY, n_imageY,
	    l_imageZ, m_imageZ, n_imageZ,
	    l_Xn, m_Xn, n_Xn,
	    l_Yn, m_Yn, n_Yn,
	    l_Zn, m_Zn, n_Zn,
	    *l_imageL, m_imageL, n_imageL,
	    *l_imageA, m_imageA, n_imageA,
	    *l_imageB, m_imageB, n_imageB;
	int ierr=0;

	int minlhs=3, maxlhs=3, minrhs=6, maxrhs=6;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	GetRhsVar(1, "d", &m_imageX, &n_imageX, &l_imageX);
	GetRhsVar(2, "d", &m_imageY, &n_imageY, &l_imageY);
	GetRhsVar(3, "d", &m_imageZ, &n_imageZ, &l_imageZ);
	GetRhsVar(4, "d", &m_Xn, &n_Xn, &l_Xn);
	GetRhsVar(5, "d", &m_Yn, &n_Yn, &l_Yn);
	GetRhsVar(6, "d", &m_Zn, &n_Zn, &l_Zn);

	xyz2lab(stk(l_imageX), stk(l_imageY), stk(l_imageZ), m_imageX, n_imageX, *stk(l_Xn), *stk(l_Yn), *stk(l_Zn), &l_imageL, &l_imageA, &l_imageB, &ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(7,"d", &m_imageX, &n_imageX, &l_imageL);
	CreateVarFromPtr(8,"d", &m_imageX, &n_imageX, &l_imageA);
	CreateVarFromPtr(9,"d", &m_imageX, &n_imageX, &l_imageB);

	if (l_imageL) {
		free(l_imageL);
	}
	if (l_imageA) {
		free(l_imageA);
	}
	if (l_imageB) {
		free(l_imageB);
	}

	LhsVar(1)=7;
	LhsVar(2)=8;
	LhsVar(3)=9;

	return(0);
}

/******************************************************************************/
/* L*a*b* image computing                                                     */
/******************************************************************************/
/* Creation : 30/04/2002 (S. BONDU)                                           */
/* Last modification : 30/04/2002 (V. COUVERT)                                */
/******************************************************************************/
int xyz2lab(double *pX, double *pY, double *pZ, int nWidth, int nHeight,
			double dXn, double dYn, double dZn, double **pL, double **pA,
			double **pB, int *ierr)
{
	int i, j;
	double fx, fy, fz;

	/* Memory allocation */
	if (!(*pL = (double*)calloc(1, sizeof(double)*nWidth*nHeight)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*pA = (double*)calloc(1, sizeof(double)*nWidth*nHeight)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*pB = (double*)calloc(1, sizeof(double)*nWidth*nHeight)))
	{
		*ierr=1;
		return(-1);
	}

	/* Main loop */
	for(i=0;i<nWidth;i++)
	{
		for(j=0;j<nHeight;j++)
		{
			if((pX[nHeight*i + j] / dXn)<=0.008856)
			{
				fx = 7.787 * (pX[nHeight*i + j] / dXn) + 16.0 / 116.0;
			}
			else
			{
				fx = pow((pX[nHeight*i + j] / dXn), 1.0/3.0);
			}
			if((pY[nHeight*i + j] / dYn)<=0.008856)
			{
				fy = 7.787 * (pY[nHeight*i + j] / dYn) + 16.0 / 116.0;
			}
			else
			{
				fy = pow((pY[nHeight*i + j] / dYn), 1.0/3.0);
			}
			if((pZ[nHeight*i + j] / dZn)<=0.008856)
			{
				fz = 7.787 * (pZ[nHeight*i + j] / dZn) + 16.0 / 116.0;
			}
			else
			{
				fz = pow((pZ[nHeight*i + j] / dZn), 1.0/3.0);
			}
			(*pL)[nHeight*i + j] = 116.0 * fy - 16.0;
			(*pA)[nHeight*i + j] = 500.0 * (fx - fy);
			(*pB)[nHeight*i + j] = 200.0 * (fy - fz);
		}
	}
	*ierr = 0;
	return (0);
}


